#!/usr/bin/python3

import argparse
import configparser
import pathlib

from . import validate
from .validate import LinuxBinary, PlatformBinary, WindowsBinary


def main() -> None:
    parser = argparse.ArgumentParser()

    parser.add_argument("validation_path", metavar="config", type=pathlib.Path)

    args = parser.parse_args()

    validation_files = set()
    if args.validation_path.is_dir():
        validation_files.update(args.validation_path.rglob("*.ini"))
    else:
        validation_files.add(args.validation_path)


    linux_target = LinuxBinary(pathlib.Path("server_srv.so"))
    windows_target = WindowsBinary(pathlib.Path("server.dll"))

    linux_target.path = pathlib.Path("232250/tf/bin/server_srv.so")
    windows_target.path = pathlib.Path("232250/tf/bin/server.dll")

    candidate_targets: list[PlatformBinary] = [linux_target, windows_target]
    for validation_file in sorted(validation_files):
        header = validation_file.stem
        if args.validation_path.is_dir():
            header = str(
                pathlib.PurePosixPath(validation_file.relative_to(args.validation_path))
            )
        print(header)

        config = configparser.ConfigParser()
        config.read(validation_file)

        entries = validate.read_config(config)

        for name, entry in entries.items():
            target = entry.get_target_match(candidate_targets)
            try:
                assert target, f"No binary match for {entry.target}"
                for key, result in entry.process(target).items():
                    print("- [OK]", key.substitute(name=name), "=", result)
            except Exception as e:
                print("- [FAIL]", name, f"({type(e).__name__}):", e)
                # traceback.print_exception(e)


if __name__ == "__main__":
    main()
